@@ -1,16 +1,19 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010-2014 -- leonerd@leonerd.org.uk
 
 package IO::Socket::IP;
+# $VERSION needs to be set before  use base 'IO::Socket'
+#  - https://rt.cpan.org/Ticket/Display.html?id=92107
+BEGIN {
+   $VERSION = '0.27';
+}
 
 use strict;
 use warnings;
 use base qw( IO::Socket );
 
-our $VERSION = '0.24';
-
 use Carp;
 
 use Socket 1.97 qw(
@@ -22,7 +25,7 @@ use Socket 1.97 qw(
    IPPROTO_IPV6 IPV6_V6ONLY
    NI_DGRAM NI_NUMERICHOST NI_NUMERICSERV NIx_NOHOST NIx_NOSERV
    SO_REUSEADDR SO_REUSEPORT SO_BROADCAST SO_ERROR
-   SOCK_DGRAM SOCK_STREAM 
+   SOCK_DGRAM SOCK_STREAM
    SOL_SOCKET
 );
 my $AF_INET6 = eval { Socket::AF_INET6() }; # may not be defined
@@ -58,8 +61,7 @@ my $IPv6_re = do {
 
 =head1 NAME
 
-C<IO::Socket::IP> - A drop-in replacement for C<IO::Socket::INET> supporting
-both IPv4 and IPv6
+C<IO::Socket::IP> - Family-neutral IP socket supporting both IPv4 and IPv6
 
 =head1 SYNOPSIS
 
@@ -80,7 +82,7 @@ both IPv4 and IPv6
 =head1 DESCRIPTION
 
 This module provides a protocol-independent way to use IPv4 and IPv6 sockets,
-as a drop-in replacement for L<IO::Socket::INET>. Most constructor arguments
+intended as a replacement for L<IO::Socket::INET>. Most constructor arguments
 and methods are provided in a backward-compatible way. For a list of known
 differences, see the C<IO::Socket::INET> INCOMPATIBILITES section below.
 
@@ -91,7 +93,7 @@ falling back to IPv4-only on systems which don't.
 
 =head1 REPLACING C<IO::Socket> DEFAULT BEHAVIOUR
 
-By placing C<-register> in the import list, C<IO::Socket> uses
+By placing C<-register> in the import list, L<IO::Socket> uses
 C<IO::Socket::IP> rather than C<IO::Socket::INET> as the class that handles
 C<PF_INET>.  C<IO::Socket> will also use C<IO::Socket::IP> rather than
 C<IO::Socket::INET6> to handle C<PF_INET6>, provided that the C<AF_INET6>
@@ -286,15 +288,10 @@ If your platform does not support disabling this option but you still want to
 listen for both C<AF_INET> and C<AF_INET6> connections you will have to create
 two listening sockets, one bound to each protocol.
 
-=item Timeout
-
-This C<IO::Socket::INET>-style argument is not currently supported. See the
-C<IO::Socket::INET> INCOMPATIBILITES section below.
-
 =item MultiHomed
 
 This C<IO::Socket::INET>-style argument is ignored, except if it is defined
-but false. See the C<IO::Socket::INET> INCOMPATIBILITES section below. 
+but false. See the C<IO::Socket::INET> INCOMPATIBILITES section below.
 
 However, the behaviour it enables is always performed by C<IO::Socket::IP>.
 
@@ -311,6 +308,12 @@ C<SOCK_STREAM> and C<IPPROTO_TCP> respectively will be set, to maintain
 compatibility with C<IO::Socket::INET>. Other named arguments that are not
 recognised are ignored.
 
+If neither C<Family> nor any hosts or addresses are passed, nor any
+C<*AddrInfo>, then the constructor has no information on which to decide a
+socket family to create. In this case, it performs a C<getaddinfo> call with
+the C<AI_ADDRCONFIG> flag, no host name, and a service name of C<"0">, and
+uses the family of the first returned result.
+
 If the constructor fails, it will set C<$@> to an appropriate error message;
 this may be from C<$!> or it may be some other string; not every failure
 necessarily has an associated C<errno> value.
@@ -324,7 +327,7 @@ behaviour given in the C<PeerHost> AND C<LocalHost> PARSING section below.
 
 =cut
 
-sub new 
+sub new
 {
    my $class = shift;
    my %arg = (@_ == 1) ? (PeerHost => $_[0]) : @_;
@@ -354,9 +357,9 @@ sub configure
       my $host    = $type . 'Host';
       my $service = $type . 'Service';
 
-      if (exists $arg->{$host} && !exists $arg->{$service}) {
-         defined $arg->{$host} or next;
+      if( defined $arg->{$host} ) {
          ( $arg->{$host}, my $s ) = $self->split_addr( $arg->{$host} );
+         # IO::Socket::INET compat - *Host parsed port always takes precedence
          $arg->{$service} = $s if defined $s;
       }
    }
@@ -512,8 +515,21 @@ sub _configure
       }
    }
 
-   if( !@infos and defined $hints{family} ) {
+   if( !@infos ) {
       # If there was a Family hint then create a plain unbound, unconnected socket
+      # If there wasn't, use getaddrinfo()'s AI_ADDRCONFIG side-effect to guess a
+      # suitable family first.
+      if( !defined $hints{family} ) {
+         my ( $err, $addrinfo ) = getaddrinfo( "", "0", \%hints );
+         if( $err ) {
+            $@ = "$err";
+            $! = EINVAL;
+            return;
+         }
+
+         $hints{family} = $addrinfo->{family};
+      }
+
       @infos = ( {
          family   => $hints{family},
          socktype => $hints{socktype},
@@ -928,8 +944,7 @@ To support the C<IO::Socket::INET> API, the host and port information may be
 passed in a single string rather than as two separate arguments.
 
 If either C<LocalHost> or C<PeerHost> (or their C<...Addr> synonyms) have any
-of the following special forms, and C<LocalService> or C<PeerService> (or
-their C<...Port> synonyms) are absent, special parsing is applied.
+of the following special forms then special parsing is applied.
 
 The value of the C<...Host> argument will be split to give both the hostname
 and port (or service name):
@@ -950,6 +965,10 @@ service name and number, in a form such as
 In this case, the name (C<http>) will be tried first, but if the resolver does
 not understand it then the port number (C<80>) will be used instead.
 
+If the C<...Host> argument is in this special form and the corresponding
+C<...Service> or C<...Port> argument is also defined, the one parsed from
+the C<...Host> argument will take precedence and the other will be ignored.
+
 =head2 ( $host, $port ) = IO::Socket::IP->split_addr( $addr )
 
 Utility method that provides the parsing functionality described above.
@@ -1048,8 +1067,6 @@ sub configure
 
 =item *
 
-The C<Timeout> constructor argument is currently not recognised.
-
 The behaviour enabled by C<MultiHomed> is in fact implemented by
 C<IO::Socket::IP> as it is required to correctly support searching for a
 useable address from the results of the C<getaddrinfo(3)> call. The
